# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import

from anaconda_project.frontend import Frontend


class FakeFrontend(Frontend):
    def __init__(self):
        super(FakeFrontend, self).__init__()
        self.logs = []
        self.errors = []

    def info(self, message):
        self.logs.append(message)

    def error(self, message):
        self.errors.append(message)

    def reset(self):
        self.logs = []
        self.errors = []



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function, unicode_literals

from anaconda_project.internal.slugify import slugify


def test_should_be_unchanged():
    s = "abcdefgxyz_ABCDEFGXYZ-0123456789"
    assert s == slugify(s)


def test_replace_spaces():
    assert "a-b" == slugify("a b")


def test_replace_unicode():
    assert "-" == slugify(u"🌟")


def test_replace_specials():
    assert "-----------------" == slugify("!@#$%^&*()<>\"':/\\")


def test_replace_bytes():
    assert "-" == slugify(u"🌟".encode('utf-8'))



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2021, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function

import subprocess

from anaconda_project.docker import build_image

try:
    FileNotFoundError  # noqa
except NameError:
    # python 2
    FileNotFoundError = OSError


def test_build_image_pass(monkeypatch):
    def mock_check_call(*args, **kwargs):
        return

    monkeypatch.setattr('subprocess.check_call', mock_check_call)

    status = build_image('.', 'tag', 'default')
    assert status


def test_build_image_extra_args(monkeypatch):
    def mock_check_call(*args, **kwargs):
        return

    monkeypatch.setattr('subprocess.check_call', mock_check_call)

    status = build_image('.', 'tag', 'default', build_args={'-f': 'Dockerfile'})
    assert status


def test_build_image_failed(monkeypatch):
    def mock_check_call(*args, **kwargs):
        raise subprocess.CalledProcessError(1, 's2i', 'failed to build')

    monkeypatch.setattr('subprocess.check_call', mock_check_call)

    status = build_image('.', 'tag', 'default')
    assert len(status.errors) == 1


def test_build_image_not_found(monkeypatch):
    def mock_check_call(*args, **kwargs):
        raise FileNotFoundError

    monkeypatch.setattr('subprocess.check_call', mock_check_call)

    status = build_image('.', 'tag', 'default')
    assert len(status.errors) == 1



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
"""Command-line-specific project load utilities."""
from __future__ import absolute_import, print_function

import sys

from anaconda_project.project import Project
from anaconda_project.frontend import Frontend

import anaconda_project.internal.cli.console_utils as console_utils


class CliFrontend(Frontend):
    def __init__(self):
        """Frontend for printing."""
        super(CliFrontend, self).__init__()

    def info(self, message):
        print(message)

    def error(self, message):
        print(message, file=sys.stderr)

    def partial_info(self, data):
        sys.stdout.write(data)
        sys.stdout.flush()

    def partial_error(self, data):
        sys.stderr.write(data)
        sys.stderr.flush()


def load_project(dirname):
    """Load a Project, fixing it if needed and possible."""
    project = Project(dirname, frontend=CliFrontend(), must_exist=True)

    # No sense in engaging the user if we cannot achieve a fixed state.
    if project.unfixable_problems:
        return project

    if console_utils.stdin_is_interactive():
        regressions = 0
        problems = project.fixable_problems
        while problems and regressions < 3:
            # Instead of looping through the problems in the list, we
            # fix only the first one and refresh the list. This allows
            # us to detect when fixing one problem impacts another,
            # positively or negatively.
            problem = problems[0]
            print(problem.text)
            should_fix = console_utils.console_ask_yes_or_no(problem.fix_prompt, default=False)
            if not should_fix:
                break
            problem.fix(project)
            project.use_changes_without_saving()
            o_problems, problems = problems, project.fixable_problems
            # If the number of problems doesn't decrease as a result of
            # fixing a problem, it suggests some sort of negative cycle.
            # We can't reliably detect a cycle, so instead we simply let
            # this happen 3 times before we give up.
            regressions += (len(problems) >= len(o_problems))
        if not problems:
            project.save()

    return project



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2019, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function

from anaconda_project.internal.cli.main import _parse_args_and_run_subcommand
from anaconda_project.internal.test.tmpfile_utils import with_directory_contents_completing_project_file
from anaconda_project.internal.simple_status import SimpleStatus


def _monkeypatch_download(monkeypatch):
    params = {}

    def mock_download(*args, **kwargs):
        params['args'] = args
        params['kwargs'] = kwargs
        return SimpleStatus(success=True, description="Yay")

    monkeypatch.setattr('anaconda_project.project_ops.download', mock_download)
    return params


def test_download(capsys, monkeypatch):
    _monkeypatch_download(monkeypatch)

    def check(dirname):
        code = _parse_args_and_run_subcommand(['anaconda-project', 'download', 'fake_user/fake_project'])
        assert code == 0

        out, err = capsys.readouterr()
        assert 'Yay\n' == out
        assert '' == err

    with_directory_contents_completing_project_file(dict(), check)


def test_download_no_username(capsys, monkeypatch):
    _monkeypatch_download(monkeypatch)

    def check(dirname):
        code = _parse_args_and_run_subcommand(['anaconda-project', 'download', 'fake_project'])
        assert code == 0

        out, err = capsys.readouterr()
        assert 'Yay\n' == out
        assert '' == err

    with_directory_contents_completing_project_file(dict(), check)


def test_download_no_unpack(capsys, monkeypatch):
    params = _monkeypatch_download(monkeypatch)

    def check(dirname):
        code = _parse_args_and_run_subcommand(['anaconda-project', 'download', 'fake_user/fake_project', '--no-unpack'])
        assert code == 0

        out, err = capsys.readouterr()
        assert 'Yay\n' == out
        assert '' == err

        assert not params['kwargs']['unpack']

    with_directory_contents_completing_project_file(dict(), check)


def test_download_parent_dir(capsys, monkeypatch):
    params = _monkeypatch_download(monkeypatch)

    def check(dirname):
        code = _parse_args_and_run_subcommand(
            ['anaconda-project', 'download', 'fake_user/fake_project', '--parent_dir', '.'])
        assert code == 0

        out, err = capsys.readouterr()
        assert 'Yay\n' == out
        assert '' == err

        assert params['kwargs']['parent_dir'] == '.'

    with_directory_contents_completing_project_file(dict(), check)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function

import codecs
import os

from anaconda_project.internal.ziputils import unpack_zip
from anaconda_project.internal.test.tmpfile_utils import (with_directory_contents, with_tmp_zipfile)


def test_unzip_single_file_different_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')  # different name from what's in the zip
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert codecs.open(os.path.join(target_path, 'foo'), 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile(dict(foo="hello world\n"), do_test)


def test_unzip_single_file_same_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'foo')  # same name as what's in the zip
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isfile(target_path)
        assert codecs.open(target_path, 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile(dict(foo="hello world\n"), do_test)


def test_unzip_two_files():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert codecs.open(os.path.join(target_path, 'foo'), 'r', 'utf-8').read() == "hello world\n"
        assert codecs.open(os.path.join(target_path, 'bar'), 'r', 'utf-8').read() == "goodbye world\n"

    with_tmp_zipfile(dict(foo="hello world\n", bar="goodbye world\n"), do_test)


def test_unzip_one_directory_different_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')  # different name from what's in the zip
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert os.path.isdir(os.path.join(target_path, 'foo'))
        assert codecs.open(os.path.join(target_path, 'foo', 'bar'), 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile({'foo/bar': "hello world\n"}, do_test)


def test_unzip_one_directory_same_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'foo')  # same name as what's in the zip
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert codecs.open(os.path.join(target_path, 'bar'), 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile({'foo/bar': "hello world\n"}, do_test)


def test_unzip_empty_zip():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert ['Zip archive was empty.'] == errors
        assert not os.path.isdir(target_path)

    with_tmp_zipfile(dict(), do_test)


# we do rename directories over directories
def test_unzip_target_already_exists_and_is_directory():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')
        os.makedirs(target_path)
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert codecs.open(os.path.join(target_path, 'foo'), 'r', 'utf-8').read() == "hello world\n"
        assert codecs.open(os.path.join(target_path, 'bar'), 'r', 'utf-8').read() == "goodbye world\n"

    with_tmp_zipfile(dict(foo="hello world\n", bar="goodbye world\n"), do_test)


# we do rename directories over directories even if the dir is inside the zip
def test_unzip_target_already_exists_and_is_directory_and_single_dir_in_zip():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')  # different name from dir in zip
        os.makedirs(target_path)
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert os.path.isdir(os.path.join(target_path, 'foo'))
        assert codecs.open(os.path.join(target_path, 'foo', 'bar'), 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile({'foo/bar': "hello world\n"}, do_test)


# we do rename directories over directories even if we strip the toplevel dir from the zip
def test_unzip_target_already_exists_and_is_directory_and_single_dir_in_zip_same_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'foo')  # same name as dir in zip
        os.makedirs(target_path)
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert codecs.open(os.path.join(target_path, 'bar'), 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile({'foo/bar': "hello world\n"}, do_test)


# we don't rename a file over a directory when we discard the zip's dir due to same name
def test_unzip_target_already_exists_and_is_directory_and_single_file_in_zip_same_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'foo')  # same name as file in zip
        os.makedirs(target_path)
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert os.path.isdir(target_path)
        assert ["%s exists and is a directory, not unzipping a plain file over it." % target_path] == errors

    with_tmp_zipfile(dict(foo="hello world\n"), do_test)


# we do rename a directory over a directory if we keep the dir since the file has a different name
def test_unzip_target_already_exists_and_is_directory_and_single_file_in_zip_different_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')  # different name from file in zip
        os.makedirs(target_path)
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isdir(target_path)
        assert codecs.open(os.path.join(target_path, 'foo'), 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile(dict(foo="hello world\n"), do_test)


# we don't rename directories over files even if the dirs are inside the zip
def test_unzip_target_already_exists_and_is_file_and_single_dir_in_zip():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')  # different name so we should keep "foo"
        with codecs.open(target_path, 'w', 'utf-8') as f:
            f.write("\n")
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert os.path.isfile(target_path)
        assert [("%s exists and isn't a directory, not unzipping a directory over it." % target_path)] == errors

    with_tmp_zipfile({'foo/bar': "hello world\n"}, do_test)


# we don't rename directories over files if the zip has no root dir
def test_unzip_target_already_exists_and_is_file():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')
        with codecs.open(target_path, 'w', 'utf-8') as f:
            f.write("\n")
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [("%s exists and isn't a directory, not unzipping a directory over it." % target_path)] == errors

    with_tmp_zipfile(dict(foo="hello world\n", bar="goodbye world\n"), do_test)


# we do rename files over files if they have the same name
def test_unzip_target_already_exists_and_is_file_and_single_file_in_zip_same_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'foo')  # same name
        with codecs.open(target_path, 'w', 'utf-8') as f:
            f.write("\n")
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [] == errors
        assert os.path.isfile(target_path)
        assert codecs.open(target_path, 'r', 'utf-8').read() == "hello world\n"

    with_tmp_zipfile(dict(foo="hello world\n"), do_test)


# we don't rename files over files if the name is different
def test_unzip_target_already_exists_and_is_file_and_single_file_in_zip_different_name():
    def do_test(zipname, workingdir):
        target_path = os.path.join(workingdir, 'boo')  # different from the file in the zip
        with codecs.open(target_path, 'w', 'utf-8') as f:
            f.write("original\n")
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [("%s exists and isn't a directory, not unzipping a directory over it." % target_path)] == errors
        assert os.path.isfile(target_path)
        assert codecs.open(target_path, 'r', 'utf-8').read() == "original\n"

    with_tmp_zipfile(dict(foo="hello world\n"), do_test)


def test_unzip_bad_zipfile():
    def do_test(workingdir):
        zipname = os.path.join(workingdir, 'foo')
        target_path = os.path.join(workingdir, 'boo')
        errors = []
        unpack_zip(zipname, target_path, errors)
        assert [('Failed to unzip %s: File is not a zip file' % zipname)] == errors

    with_directory_contents(dict(foo="not a zip file\n"), do_test)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016, Anaconda, Inc. All rights reserved.
#
# Licensed under the terms of the BSD 3-Clause License.
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------
from __future__ import absolute_import, print_function

from anaconda_project.internal.cli.main import _parse_args_and_run_subcommand
from anaconda_project.internal.test.tmpfile_utils import with_directory_contents_completing_project_file
from anaconda_project.project_file import DEFAULT_PROJECT_FILENAME


def test_clean_command_on_empty_project(capsys):
    def check(dirname):
        code = _parse_args_and_run_subcommand(['anaconda-project', 'clean', '--directory', dirname])
        assert code == 0

        out, err = capsys.readouterr()
        assert "Nothing to clean up for environment 'default'.\nCleaned.\n" == out
        assert '' == err

    with_directory_contents_completing_project_file(dict(), check)


def test_clean_command_on_invalid_project(capsys):
    def check(dirname):
        code = _parse_args_and_run_subcommand(['anaconda-project', 'clean', '--directory', dirname])
        assert code == 1

        out, err = capsys.readouterr()
        assert '' == out
        assert ('variables section contains wrong value type 42,' + ' should be dict or list of requirements\n' +
                'Unable to load the project.\n' + 'Failed to clean everything up.\n') in err

    with_directory_contents_completing_project_file({DEFAULT_PROJECT_FILENAME: "variables:\n  42"}, check)
